// A class for automatically loading mutators, using a .ini file instead of the commandline
//
//	Copyright (C) 2008 John "Shambler" Barrett (JBarrett847@Gmail.com or Shambler@OldUnreal.com)
//
//	This program is free software; you can redistribute and/or modify
//	it under the terms of the Open Unreal Mod License version 1.1.
//
Class MLMut extends Mutator
	config(MutatorLoader);


// The list of mutators to be immediately added; mutators in this list will have 'InitMutator' called
var array<string> Mutators;


// The list of mutators to be added after one tick; these mutators might not have 'InitMutator' called
// NOTE: Only use this list if you have so many mutators running, that the server crashes through infinite recursion.
//		If any mutators in this list require 'InitMutator', then they might break.
var array<string> DelayedMutators;

// If this is set to True, then the code will attempt to simulate 'InitMutator' for all of the 'DelayedMutator' entries
// NOTE: This is a hack, and some mutators relying upon 'InitMutator' may still break
var bool bSimulateInitMutator;

// If true, the config values are loaded per-gametype, using perobjectconfig sections in the .ini
// NOTE: If no section can be found for the current gametype, then the default configuration is used instead
var config bool bPerGameTypeConfig;

// If true, then each gametype is automatically given its own section in the .ini file, as a copy of the default configuration
var config bool bAutoCreateGameTypeConfig;

// If true, then this mutator is not visible in the server browser (but can still be filtered; may break mutator display in WebAdmin)
var config bool bDontAdvertiseMutator;


// The currently loaded configuration profile
var MLConfigProfile ConfigObject;


function InitMutator(string Options, out string ErrorMessage)
{
	local Object TempObj;
	local UTUIDataProvider_Mutator DP;
	local string MutStr, LogStr;

	// Check that the data provider for this mutator exists, and adjust the 'FriendlyName' value if necessary (based upon bDontAdvertiseMutator)
	TempObj = new(Class.Outer, "MutatorLoader") Class'Package';
	DP = new(TempObj, string(Class)) Class'UTUIDataProvider_Mutator';

	if (DP.ClassName != PathName(Class))
	{
		LogInternal("Storing data provider for '"$Class$"'", 'MutatorLoader');

		DP.ClassName = PathName(Class);

		if (!bDontAdvertiseMutator)
			Class'UnrealScriptTools14.USTools'.static.SetFriendlyName(DP, "Mutator Loader");

		Class'UnrealScriptTools14.USTools'.static.SetDescription(DP, ".ini based mutator loader");

		DP.SaveConfig();

		// If the data provider config does not exist, assume this classes config does not exist either
		SaveConfig();
	}

	if (bDontAdvertiseMutator && DP.FriendlyName != "")
	{
		Class'UnrealScriptTools14.USTools'.static.SetFriendlyName(DP, "");
		DP.SaveConfig();
	}
	else if (!bDontAdvertiseMutator && DP.FriendlyName == "")
	{
		Class'UnrealScriptTools14.USTools'.static.SetFriendlyName(DP, "Mutator Loader");
		DP.SaveConfig();
	}


	// Load the required config profile
	LoadPresetConfig();


	// Load the mutators from the 'Mutators' list
	foreach Mutators(MutStr)
	{
		WorldInfo.Game.AddMutator(MutStr, True);

		if (LogStr != "")
			LogStr $= ", ";

		LogStr $= MutStr;
	}

	if (LogStr != "")
		`log("Added Mutators:"@LogStr,, 'MutatorLoader');


	Enable('Tick');


	Super.InitMutator(Options, ErrorMessage);
}

function Tick(float DeltaTime)
{
	local Mutator LastMutator;
	local string MutStr, LogStr, Error;

	if (DelayedMutators.Length == 0)
	{
		Destroy();
		return;
	}


	// If 'bSimulateInitMutator' is true, then store a reference to the mutator at the end of the mutator chain
	if (bSimulateInitMutator)
		for (LastMutator=WorldInfo.Game.BaseMutator; LastMutator!=none; LastMutator=LastMutator.NextMutator)
			if (LastMutator.NextMutator == none)
				break;

	// Load the mutators from the 'DelayedMutators' list
	foreach DelayedMutators(MutStr)
	{
		WorldInfo.Game.AddMutator(MutStr, True);

		if (LogStr != "")
			LogStr $= ", ";

		LogStr $= MutStr;
	}

	if (LogStr != "")
		`log("Delayed Mutators:"@LogStr,, 'MutatorLoader');


	if (bSimulateInitMutator && LastMutator != none && LastMutator.NextMutator != none)
		LastMutator.NextMutator.InitMutator(WorldInfo.Game.ServerOptions, Error);


	Destroy();
}


function LoadPresetConfig()
{
	local string ConfigName;
	local array<string> KnownProfiles;
	local int i, cLen;
	local bool bDefaultProfileExists, bProfileExists;
	local MLConfigProfile DefaultConfigObj;

	// First decide upon the config profile that is to be used
	if (bPerGameTypeConfig)
		ConfigName = string(WorldInfo.Game.Class);
	else
		ConfigName = "default";


	// Then check that it exists
	if (GetPerObjectConfigSections(Class'MLConfigProfile', KnownProfiles))
	{
		ConfigName @= "";
		cLen = Len(ConfigName);

		for (i=0; i<KnownProfiles.Length; ++i)
		{
			if (bPerGameTypeConfig && Left(KnownProfiles[i], 8) ~= "default ")
				bDefaultProfileExists = True;
			else if (Left(KnownProfiles[i], cLen) ~= ConfigName)
				bProfileExists = True;

			if (bProfileExists && (bDefaultProfileExists || !bPerGameTypeConfig))
				break;
		}

		ConfigName = Left(ConfigName, cLen-1);
	}

	if (!bPerGameTypeConfig && bProfileExists)
		bDefaultProfileExists = True;


	// If it does not exist, determine whether or not it should be created, or if we should revert to default
	if (!bProfileExists)
	{
		// If the default profile doesn't exist, create it
		if (!bDefaultProfileExists)
		{
			LogInternal("Default profile did not exist, creating default profile", 'MutatorLoader');

			DefaultConfigObj = new(none, "default") Class'MLConfigProfile';
			DefaultConfigObj.SaveConfig();
		}

		if (bPerGameTypeConfig && bAutoCreateGameTypeConfig)
		{
			LogInternal("Could not find configuration profile '"$ConfigName$"', creating profile based on default profile", 'MutatorLoader');

			if (DefaultConfigObj == none)
				DefaultConfigObj = new(none, "default") Class'MLConfigProfile';

			ConfigObject = new(none, ConfigName) Class'MLConfigProfile';

			DefaultConfigObj.TransferProperties(Self);
			ConfigObject.SaveProperties(Self);
		}
		else
		{
			if (bPerGameTypeConfig)
				LogInternal("Could not find configuration profile '"$ConfigName$"', loading default configuration profile", 'MutatorLoader');
			else
				LogInternal("Loading configuration profile '"$ConfigName$"'", 'MutatorLoader');

			if (DefaultConfigObj != none)
				ConfigObject = DefaultConfigObj;
			else
				ConfigObject = new(none, "default") Class'MLConfigProfile';

			ConfigObject.TransferProperties(Self);
		}
	}
	else
	{
		LogInternal("Loading configuration profile '"$ConfigName$"'", 'MutatorLoader');

		ConfigObject = new(none, ConfigName) Class'MLConfigProfile';
		ConfigObject.TransferProperties(Self);
	}
}


defaultproperties
{
	bAutoCreateGameTypeConfig=True
}
